@@ -65,7 +65,9 @@ class Sync(Base):
         self.setting = document.get('setting')
         super().__init__(document.get('database', self.index), **params)
         self.es = ElasticHelper()
-        self.__name = f"{self.database}_{self.index}"
+        self.__name = re.sub(
+            '[^0-9a-zA-Z_]+', '', f"{self.database}_{self.index}"
+        )
         self._checkpoint = None
         self._truncate = False
         self.verbose = verbose
@@ -206,8 +208,8 @@ class Sync(Base):
 
         for row in rows:
             if (
-                re.search(r'BEGIN', row.data) or
-                re.search(r'COMMIT', row.data)
+                re.search(r'^BEGIN', row.data) or
+                re.search(r'^COMMIT', row.data)
             ):
                 continue
             _rows.append(row)
@@ -813,7 +815,12 @@ class Sync(Base):
                 i += 1
                 continue
 
-            conn.poll()
+            try:
+                conn.poll()
+            except psycopg2.OperationalError as e:
+                logger.fatal(f'OperationalError: {e}')
+                os._exit(-1)
+
             while conn.notifies:
                 notification = conn.notifies.pop(0)
                 payload = json.loads(notification.payload)
