@@ -7,7 +7,7 @@ import pytest
 from pgsync.base import subtransactions
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, search
+from .helpers.utils import assert_resync_empty, search, truncate_slots
 
 
 @pytest.mark.usefixtures('table_creator')
@@ -29,7 +29,7 @@ class TestNestedChildren(object):
         language_cls,
         book_subject_cls,
         subject_cls,
-        bookshelf_cls,
+        book_shelf_cls,
         shelf_cls,
     ):
         session = sync.session
@@ -207,11 +207,11 @@ class TestNestedChildren(object):
         ]
 
         book_shelves = [
-            bookshelf_cls(id=1, book=books[0], shelf=shelves[0]),
-            bookshelf_cls(id=2, book=books[1], shelf=shelves[0]),
-            bookshelf_cls(id=3, book=books[2], shelf=shelves[1]),
-            bookshelf_cls(id=4, book=books[0], shelf=shelves[1]),
-            bookshelf_cls(id=5, book=books[1], shelf=shelves[1]),
+            book_shelf_cls(id=1, book=books[0], shelf=shelves[0]),
+            book_shelf_cls(id=2, book=books[1], shelf=shelves[0]),
+            book_shelf_cls(id=3, book=books[2], shelf=shelves[1]),
+            book_shelf_cls(id=4, book=books[0], shelf=shelves[1]),
+            book_shelf_cls(id=5, book=books[1], shelf=shelves[1]),
         ]
 
         with subtransactions(session):
@@ -268,7 +268,7 @@ class TestNestedChildren(object):
                     language_cls.__table__.name,
                     book_subject_cls.__table__.name,
                     subject_cls.__table__.name,
-                    bookshelf_cls.__table__.name,
+                    book_shelf_cls.__table__.name,
                     shelf_cls.__table__.name,
                 ]
             )
@@ -640,7 +640,7 @@ class TestNestedChildren(object):
         language_cls,
         book_subject_cls,
         subject_cls,
-        bookshelf_cls,
+        book_shelf_cls,
         shelf_cls
     ):
         """Test insert a new root item."""
@@ -719,8 +719,8 @@ class TestNestedChildren(object):
         ]
 
         book_shelves = [
-            bookshelf_cls(id=6, book=books[0], shelf=shelves[0]),
-            bookshelf_cls(id=7, book=books[0], shelf=shelves[1]),
+            book_shelf_cls(id=6, book=books[0], shelf=shelves[0]),
+            book_shelf_cls(id=7, book=books[0], shelf=shelves[1]),
         ]
 
         document = {
@@ -948,7 +948,7 @@ class TestNestedChildren(object):
         data,
         nodes,
         book_cls,
-        bookshelf_cls,
+        book_shelf_cls,
         book_language_cls,
         book_subject_cls,
         book_author_cls
@@ -974,8 +974,8 @@ class TestNestedChildren(object):
         def poll_db():
             with subtransactions(session):
                 session.execute(
-                    bookshelf_cls.__table__.delete().where(
-                        bookshelf_cls.__table__.c.book_isbn == 'abc'
+                    book_shelf_cls.__table__.delete().where(
+                        book_shelf_cls.__table__.c.book_isbn == 'abc'
                     )
                 )
                 session.execute(
@@ -1003,8 +1003,12 @@ class TestNestedChildren(object):
         with mock.patch('pgsync.sync.Sync.poll_redis', side_effect=poll_redis):
             with mock.patch('pgsync.sync.Sync.poll_db', side_effect=poll_db):
                 with mock.patch('pgsync.sync.Sync.pull', side_effect=pull):
-                    sync.receive()
-                    sync.es.refresh('testdb')
+                    with mock.patch(
+                        'pgsync.sync.Sync.truncate_slots',
+                        side_effect=truncate_slots,
+                    ):
+                        sync.receive()
+                        sync.es.refresh('testdb')
 
         txmin = sync.checkpoint
         docs = [
@@ -1174,7 +1178,7 @@ class TestNestedChildren(object):
                         name='Nigeria',
                         continent=continent_cls(
                             id=6,
-                            name='Africa'
+                            name='Africa',
                         )
                     )
                 )
@@ -1930,3 +1934,233 @@ class TestNestedChildren(object):
     def test_delete_nonthrough_child_op(self, sync, data):
         # delete a new non-through child with op
         pass
+
+
+    @pytest.fixture(scope='function')
+    def nodes2(self):
+        return [{
+            "table": "book",
+            "columns": [
+                "isbn",
+                "title",
+                "description"
+            ],
+            "children": [
+                {
+                    "table": "publisher",
+                    "columns": [
+                        "name",
+                        "id"
+                    ],
+                    "label": "publisher_label",
+                    "relationship": {
+                        "variant": "object",
+                        "type": "one_to_one"
+                    },
+                    "children": [
+
+                    ],
+                    "transform": {
+                    }
+                },
+                {
+                    "table": "book_language",
+                    "columns": [
+                        "book_isbn",
+                        "language_id"
+                    ],
+                    "label": "book_languages",
+                    "relationship": {
+                        "variant": "object",
+                        "type": "one_to_many"
+                    }
+                },
+                {
+                    "table": "author",
+                    "columns": [
+                        "id", "name"
+                    ],
+                    "label": "authors",
+                    "relationship": {
+                        "type": "one_to_many",
+                        "variant": "object",
+                        "through_tables": [
+                            "book_author"
+                        ]
+                    },
+                    "children": [
+                        {
+                            "table": "city",
+                            "columns": [
+                                "name",
+                                "id"
+                            ],
+                            "label": "city_label",
+                            "relationship": {
+                                "variant": "object",
+                                "type": "one_to_one"
+                            },
+                            "children": [
+                                {
+                                    "table": "country",
+                                    "columns": [
+                                        "name",
+                                        "id"
+                                    ],
+                                    "label": "country_label",
+                                    "relationship": {
+                                        "variant": "object",
+                                        "type": "one_to_many"
+                                    },
+                                    "children": [
+                                        {
+                                            "table": "continent",
+                                            "columns": [
+                                                "name"
+                                            ],
+                                            "label": "continent_label",
+                                            "relationship": {
+                                                "variant": "object",
+                                                "type": "one_to_one"
+                                            }
+                                        }
+                                    ]
+                                }
+                            ]
+                        }
+                    ]
+                },
+                {
+                    "table": "language",
+                    "label": "languages",
+                    "columns": [
+                        "code"
+                    ],
+                    "relationship": {
+                        "type": "one_to_many",
+                        "variant": "scalar",
+                        "through_tables": [
+                            "book_language"
+                        ]
+                    }
+                },
+                {
+                    "table": "subject",
+                    "label": "subjects",
+                    "columns": [
+                        "name"
+                    ],
+                    "relationship": {
+                        "type": "one_to_many",
+                        "variant": "scalar",
+                        "through_tables": [
+                            "book_subject"
+                        ]
+                    }
+                }
+            ]
+        }]
+
+    def test_insert_deep_nested_nonthrough_child_noop(
+        self,
+        data,
+        nodes,
+        city_cls,
+        country_cls,
+        continent_cls,
+    ):
+        """insert a new deep nested non-through child with noop."""
+        country = country_cls(
+            id=5,
+            name='Marioworld',
+            continent=continent_cls(
+                id=6,
+                name='Bowser Land'
+            )
+        )
+
+        document = {
+            'index': 'testdb',
+            'nodes': nodes,
+        }
+        # sync first to add the initial document
+        sync = Sync(document)
+        sync.sync()
+        session = sync.session
+        sync.es.refresh('testdb')
+
+        def pull():
+            txmin = sync.checkpoint
+            txmax = sync.txid_current
+            sync.logical_slot_changes(txmin=txmin, txmax=txmax)
+
+        def poll_redis():
+            return []
+
+        def poll_db():
+            with subtransactions(session):
+                session.add(country)
+
+        with mock.patch('pgsync.sync.Sync.poll_redis', side_effect=poll_redis):
+            with mock.patch('pgsync.sync.Sync.poll_db', side_effect=poll_db):
+                with mock.patch('pgsync.sync.Sync.pull', side_effect=pull):
+                    with mock.patch(
+                        'pgsync.sync.Sync.truncate_slots',
+                        side_effect=truncate_slots,
+                    ):
+                        sync.receive()
+                        sync.es.refresh('testdb')
+
+        txmin = sync.checkpoint
+        docs = [
+            doc for doc in sync._sync(
+                nodes, 'testdb', txmin=txmin
+            )
+        ]
+        assert docs == []
+        docs = search(sync.es, 'testdb')
+        assert_resync_empty(sync, nodes, 'testdb')
+
+
+    def test_insert_deep_nested_fk_nonthrough_child_op(
+        self,
+        data,
+        nodes,
+        city_cls,
+        country_cls,
+        continent_cls,
+    ):
+        """insert a new deep nested non-through fk child with op."""
+        
+
+        import pprint
+        nodes[0]['children'].append(
+            {
+                "table": "book_rating",
+                "columns": [
+                    "book_isbn",
+                    "rating",
+                ],
+                "label": "book_ratings",
+                "relationship": {
+                    "variant": "object",
+                    "type": "one_to_many",
+                }
+            },
+        )
+
+        pprint.pprint(nodes[0]['children'])
+        print('-' * 1000)
+
+         # {
+         #            "table": "book_language",
+         #            "columns": [
+         #                "book_isbn",
+         #                "language_id"
+         #            ],
+         #            "label": "book_languages",
+         #            "relationship": {
+         #                "variant": "object",
+         #                "type": "one_to_many"
+         #            }
+         #        },
