@@ -11,7 +11,7 @@ from pgsync.exc import (
 )
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, search
+from .helpers.utils import assert_resync_empty, search, truncate_slots
 
 
 @pytest.mark.usefixtures('table_creator')
@@ -537,8 +537,12 @@ class TestRoot(object):
         with mock.patch('pgsync.sync.Sync.poll_redis', side_effect=poll_redis):
             with mock.patch('pgsync.sync.Sync.poll_db', side_effect=poll_db):
                 with mock.patch('pgsync.sync.Sync.pull', side_effect=pull):
-                    sync.receive()
-                    sync.es.refresh('testdb')
+                    with mock.patch(
+                        'pgsync.sync.Sync.truncate_slots',
+                        side_effect=truncate_slots,
+                    ):
+                        sync.receive()
+                        sync.es.refresh('testdb')
 
         docs = search(sync.es, 'testdb')
 
@@ -769,8 +773,12 @@ class TestRoot(object):
         with mock.patch('pgsync.sync.Sync.poll_redis', side_effect=poll_redis):
             with mock.patch('pgsync.sync.Sync.poll_db', side_effect=poll_db):
                 with mock.patch('pgsync.sync.Sync.pull', side_effect=pull):
-                    sync.receive()
-                    sync.es.refresh('testdb')
+                    with mock.patch(
+                        'pgsync.sync.Sync.truncate_slots',
+                        side_effect=truncate_slots,
+                    ):
+                        sync.receive()
+                        sync.es.refresh('testdb')
 
         docs = search(sync.es, 'testdb')
 
@@ -850,8 +858,12 @@ class TestRoot(object):
         with mock.patch('pgsync.sync.Sync.poll_redis', side_effect=poll_redis):
             with mock.patch('pgsync.sync.Sync.poll_db', side_effect=poll_db):
                 with mock.patch('pgsync.sync.Sync.pull', side_effect=pull):
-                    sync.receive()
-                    sync.es.refresh('testdb')
+                    with mock.patch(
+                        'pgsync.sync.Sync.truncate_slots',
+                        side_effect=truncate_slots,
+                    ):
+                        sync.receive()
+                        sync.es.refresh('testdb')
 
         docs = search(sync.es, 'testdb')
 
